import time
import json

from utils.rabbitmq import RabbitMQPublisher
import logging
logger = logging.getLogger("connectors")

def publish_audit_log(data):
    rmq = RabbitMQPublisher()
    rmq.publish("texchange.cyops.sdk",
                "key.sdk.audit",
                body=json.dumps(data),
                content_type="application/json"
                )

def audit_connector_action(audit_data, status):
    try:
        user_iri = audit_data.get("user_iri")
        user_id = user_iri.split('/')[4]

        record_iri = audit_data.get("record_iri")
        record_type = record_iri.split('/')[3]
        record_uuid = record_iri.split('/')[4]
        audit_data['data']['status'] = status

        audit_data = {
            "operation": "Executed Action",
            "component": "sdk",
            "entityType": record_type,
            "entityUuid": record_uuid,
            "userId": user_id,
            "source": audit_data.get("source"),
            "transactionDate": int(round(time.time() * 1000)),
            "data" : audit_data['data']
        }
        publish_audit_log([audit_data])
    except Exception as e:
        logger.error("Error while auditing connector action execution %s", str(e))

