import os
import re

from django.conf import settings
from django.core.management.base import BaseCommand
from connectors.views import import_connector
from connectors.models import Connector


class Command(BaseCommand):
    help = 'Installing connectors from a given path'

    def add_arguments(self, parser):
        parser.add_argument('--path', type=str, help='Path of the connector', )
        parser.add_argument('--name', type=str, help='Name of connector', )
        parser.add_argument('--compversion', type=str, help='Compatibility Version', default=None, )

    def handle(self, *args, **options):
        try:
            path = options.get('path')
            name = options.get('name')
            compversion = options.get('compversion')
            filename, file_extension = os.path.splitext(path)
            if file_extension == '.tgz':
                isReserved = False
                if filename in settings.CONNECTORS_RESERVED:
                    isReserved = True
                replace = is_replace(name, compversion)
                import_connector(path, replace, isReserved)
            else:
                self.stdout.write(self.style.ERROR('Error : connector must be in tgz format'))
        except Exception as exp:
            self.stdout.write(self.style.ERROR('Error importing the connector: %s' % str(exp)))


def is_replace(connector_name, compatible_version=None):
    if not compatible_version:
        return True

    compatible_version = compatible_version.strip()

    if compatible_version == 'ALL':
        return True

    if compatible_version == 'NULL':
        return False

    if not re.compile(r"[0-9]+.[0-9]+.[0-9]+$").match(compatible_version):
        raise Exception('Incorrect Compatible Version Information')

    connectors = Connector.objects.filter(name=connector_name)
    if (connectors.count()) > 0:
        compatible_version_map = tuple(map(int, (compatible_version.split("."))))
        for connector in connectors:
            version = connector.version
            if tuple(map(int, (version.split(".")))) < compatible_version_map:
                return False
        return True
    raise Exception('Failed to validate against installed connectors')
