from rest_framework import serializers
from connectors.models import Connector, Operation, Configuration


class ConnectorConfigurationListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Configuration
        fields = ('id', 'config_id', 'name',)


class ConnectorListSerializer(serializers.ModelSerializer):
    configuration = ConnectorConfigurationListSerializer(many=True, read_only=True)

    class Meta:
        model = Connector
        fields = ('id', 'name', 'version', 'label', 'category', 'active', 'system', 'icon_small', 'icon_large',
                  'description', 'config_count', 'status', 'install_result', 'configuration', 'ingestion_supported', 'tags')


class ConnectorConfigurationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Configuration
        fields = ('id', 'config_id', 'name', 'default', 'config', 'connector', 'status')


class ConnectorOperationSerializer(serializers.ModelSerializer):
    def to_representation(self, instance):
        ret = super().to_representation(instance)
        metadata_fields = ret['metadata']
        for field, value in metadata_fields.items():
            if not ret.get(field, None):
                ret[field] = value
        ret.pop('metadata', None)
        return ret

    class Meta:
        model = Operation
        fields = ('__all__')


class ConnectorDetailSerializer(serializers.ModelSerializer):
    configuration = ConnectorConfigurationSerializer(many=True, read_only=True)
    operations = ConnectorOperationSerializer(many=True, read_only=True)

    def to_representation(self, instance):
        ret = super().to_representation(instance)
        ret.pop('configuration_old', None)
        ret.pop('migrate', None)
        metadata_fields = ret['metadata']
        for field, value in metadata_fields.items():
            if not ret.get(field, None):
                ret[field] = value
        ret.pop('metadata', None)
        return ret

    class Meta:
        model = Connector
        fields = '__all__'
        read_only_fields = ()
